--- clamav-devel/clamd/scanner.c	Thu Apr  8 01:51:31 2004
+++ clamav-devel-jm/clamd/scanner.c	Fri Apr  9 11:18:30 2004
@@ -33,6 +33,7 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
+#include <errno.h>
 #include <clamav.h>
 
 #include "cfgparser.h"
@@ -194,7 +195,7 @@
 
 int scanstream(int odesc, unsigned long int *scanned, const struct cl_node *root, const struct cl_limits *limits, int options, const struct cfgstruct *copt)
 {
-	int ret, portscan = CL_DEFAULT_MAXPORTSCAN, sockfd, port, acceptd, tmpd, bread, retval, timeout;
+	int ret, portscan = CL_DEFAULT_MAXPORTSCAN, sockfd, port, acceptd, tmpd, bread, btread, retval, timeout;
 	long int size = 0, maxsize = 0;
 	short bound = 0;
 	const char *virname;
@@ -287,24 +288,15 @@
 	if((cpt = cfgopt(copt, "StreamMaxLength")))
 	    maxsize = cpt->numarg;
 
+	btread = sizeof(buff);
+
 	while((retval = poll_fd(acceptd, timeout)) == 1) {
-	    bread = read(acceptd, buff, sizeof(buff));
+	    bread = read(acceptd, buff, btread);
 	    if (bread <= 0) {
 		break;
 	    }
 	    size += bread;
 
-	    if(maxsize && (size + sizeof(buff)) > maxsize) {
-		shutdown(sockfd, 2);
-		close(sockfd);
-		close(acceptd);
-		mdprintf(odesc, "Size exceeded ERROR\n");
-		logg("^ScanStream: Size exceeded (stopped at %d, max: %d)\n", size, maxsize);
-		if(tmp)
-		    fclose(tmp);
-		return -1;
-	    }
-
 	    if(writen(tmpd, buff, bread) != bread) {
 		shutdown(sockfd, 2);
 		close(sockfd);
@@ -316,7 +308,20 @@
 		return -1;
 	    }
 
+	    if(maxsize && (size + btread > maxsize))
+		    btread = (maxsize - size); /* only read up to max */
+	    if (!btread) {
+		logg("^ScanStream: Size limit reached ( max: %d)\n", maxsize);
+	    	break; /* Scan what we have */
+	    }
+
 	}
+	if(retval == 0)
+            logg("!ScanStream: poll_fd timedout %d\n",timeout);
+        if(retval == -1)
+            logg("!ScanStream: poll_fd returned %s\n",strerror(errno));
+
+
 
 	lseek(tmpd, 0, SEEK_SET);
 	ret = cl_scandesc(tmpd, &virname, scanned, root, limits, options);
